#!/usr/bin/python
# -*- coding: utf-8 -*-

# manWidget.py
#
# This file is part of BotMU
#
# Author: Raphaël Perraudeau <r.perraudeau@gmail.com>
# Copyright: 2011 Raphaël Perraudeau <r.perraudeau@gmail.com>
# Licence: GNU/GPL
#
# Home page: <http://botmu.sourceforge.net/>


# Inclusions des modules:
# Natifs
import os, platform
import re
import time
import subprocess

# Pyqt4
from PyQt4.QtCore import *
from PyQt4.QtGui import *

# Perso
from commonFunctions import *
from checkUpdate import *
from httpReq import *
from options import *
from tableModel import *

# Les differentes plateforme

import Megaupload
import Youtube
import Classic


#############################################################################
class mainWidget(QWidget):

	#========================================================================
	def __init__(self, parent=None):
		super(mainWidget,self).__init__(parent)
		self.setGeometry(150, 150, 1000, 400)
		self.setMinimumSize(900, 400)

		
		##########################################################
		#                    Var Initialisation                  #
		##########################################################
		
		self.appMethod					= "gui"
		self.configDir					= getConfDir()
		self.execDir					= getExecDir()
		self.userPersonalPath 			= getUserPersonalPath()
		self.parent						= parent
		self.proxy						= ""
		self.verbose					= 0
		self.limit						= 0
		self.defaultDir					= os.path.join(self.userPersonalPath, u"Téléchargements")
		self.usedDir					= ""
		self.checkUpdate				= 1
		self.checkupdateDelay			= 0
		self.checkupdateLast			= 0
		self.telech						= {}
		self.execTime					= {}
		self.downloadSpeedByPlatform	= {}
		self.downloadedOctets			= {}
		self.lasttime					= {}
		self.timeLeft					= "?"
		self.tabledata					= []
		self.tmheader					= ['Lien', 'Nom', 'Progression']
		self.barreNull					= 0
		self.newLog						= False
		self.sendStartSignal			= False
		self.platformButton				= {}
		self.telechInit					= {}
		self.supportedPlatform			= {
			"Megaupload":[os.path.join(self.execDir, "icons/platform/megaupload.png"), "megaupload.com/?d=", 0, 8, r"[A-Z0-9]{8}", Megaupload],
			"Youtube":[os.path.join(self.execDir, "icons/platform/youtube.png"), "youtube.com/watch?v=", 1, 11, r"[a-zA-Z0-9_\-]{11}", Youtube],
			"Classique":[os.path.join(self.execDir, "icons/platform/classic.png"), "http://", 2, 0, r"[\w\d:#@%/;$()~_?\+-=\\\.&]*", Classic]
		}
		self.fileList 					= []
		self.fileListByPlatform 		= {
			"Megaupload":[],
			"Youtube":[],
			"Classique":[]
		}
		self.textToShow					= {
			"Megaupload":"",
			"Youtube":"",
			"Classique":""
		}
		self.textToShow2					= {
			"Megaupload":"",
			"Youtube":"",
			"Classique":""
		}
		self.downloadSpeed 				= 0
		self.selectedPlatform = "Megaupload"
		
		
		##########################################################
		#                   Check Update/Session                 #
		##########################################################

		self.readOptions()

		##########################################################
		#                      About Values                      #
		##########################################################

		self.aboutLogo = os.path.join(self.execDir, 'icons/BotMU.png')
		self.aboutAuthor = u'Raphaël Perraudeau'
		self.aboutAlias = u'xXKirastarotHXx'
		self.aboutMail = 'r.perraudeau@gmail.com'
		self.aboutDescription = u'BotMU est un petit logiciel qui permet de lancer une liste de téléchargements via megaupload'
		self.aboutWebSite = 'http://botmu.sourceforge.net/'


		##########################################################
		#                   Window Construction                  #
		##########################################################
		
		#Platform toolbar
			#Megaupload
		self.platformButtonMegaupload = QPushButton(QIcon(self.supportedPlatform["Megaupload"][0]), "Megaupload", self)
		self.platformButtonMegaupload.clicked.connect(lambda: self.selectPlatform("Megaupload"))
			#Youtube
		self.platformButtonYoutube = QPushButton(QIcon(self.supportedPlatform["Youtube"][0]), "Youtube", self)
		self.platformButtonYoutube.clicked.connect(lambda: self.selectPlatform("Youtube"))
			#Classique
		self.platformButtonClassic = QPushButton(QIcon(self.supportedPlatform["Classique"][0]), "Classique", self)
		self.platformButtonClassic.clicked.connect(lambda: self.selectPlatform("Classique"))
		
		#Label 1
		self.label1 = QLabel(u"Liste des fichiers à télécharger:", self)

		#ListView
		self.listView = QTableView()

		#Delete Link Button
		self.delLink = QPushButton(QIcon(os.path.join(self.execDir, 'icons/deleteLink.png')), u"Supprimer", self)
		self.delLink.clicked.connect(self.delRow_m)
		self.delLink.setEnabled(False)

		#Edit Link Button
		self.editLink = QPushButton(QIcon(os.path.join(self.execDir, 'icons/edit.png')), u"Modifier", self)
		self.editLink.clicked.connect(self.editValue_m)
		self.editLink.setEnabled(False)

		#Label 2
		self.space2 = QLabel(u"", self)
		self.label2 = QLabel(u"Emplacement sur le disque:", self)

		#Download Directory
		if(self.usedDir!=""):
			dirToUse = self.usedDir
		else:
			dirToUse = self.defaultDir
		self.fichier = QLineEdit(u"%s" % dirToUse, self)

		#File Selection Button
		self.fichierSelect = QPushButton(u"Parcourir", self)
		self.fichierSelect.clicked.connect(self.selectDir_m)

		#File Selection Button
		self.showDirectory = QPushButton(u"Ouvrir le dossier", self)
		self.showDirectory.clicked.connect(self.openDirInNavigator_m)

		#Progress Barre
		self.barre = QProgressBar(self)

		self.barre.reset()
		self.barre.setRange(0, 100)
		self.barre.setValue(0)
		
		#Information Label
		self.spaceInfo = QLabel(u"", self)
		self.labelInfo = QLabel(u"En attente de lancement", self)
		self.labelInfo2 = QLabel(u"", self)

		#Start Button 
		self.depart = QPushButton(QIcon(os.path.join(self.execDir, 'icons/start.png')), u"  Départ", self)
		self.depart.clicked.connect(self.depart_m)
 
 		#Stop Button
		self.stop = QPushButton(QIcon(os.path.join(self.execDir, 'icons/stop.png')), u"  Stop", self)
		self.stop.clicked.connect(self.stop_m)

		#Tray icon
		self.trayIcon = QSystemTrayIcon(QIcon(os.path.join(self.execDir, "BotMU.ico")))
		self.trayIcon.setToolTip("BotMU")
		self.trayIcon.show()

		##########################################################
		#                    Window Position                     #
		##########################################################

		posit = QGridLayout()
		
		
		posit.addWidget(self.platformButtonMegaupload, 0, 0)
		posit.addWidget(self.platformButtonYoutube, 0, 1)
		posit.addWidget(self.platformButtonClassic, 0, 2)

		
		
		posit.addWidget(self.label1, 1, 0)
		posit.addWidget(self.listView, 2, 0, 1, 5)
		posit.addWidget(self.delLink, 3, 3)
		posit.addWidget(self.editLink, 3, 4)
		posit.addWidget(self.space2, 4, 0)
		posit.addWidget(self.label2, 5, 0)
		posit.addWidget(self.fichier, 6, 0, 1, 3)
		posit.addWidget(self.fichierSelect, 6, 3)
		posit.addWidget(self.showDirectory, 6, 4)
		posit.addWidget(self.spaceInfo, 7, 0)
		posit.addWidget(self.barre, 8, 0, 1, 5)
		posit.addWidget(self.labelInfo, 9, 1, 1, 4)
		posit.addWidget(self.labelInfo2, 10, 1, 1, 4)
		posit.addWidget(self.depart, 11, 3)
		posit.addWidget(self.stop, 11, 4)
		self.setLayout(posit)

		self.checkIfLastSessionSaved()
		self.selectPlatform(self.selectedPlatform)
		

		if (self.checkUpdate != 0):
			checkUpdate = checkIfUpdate(parent=self)
			self.connect(checkUpdate, SIGNAL("showLastUpdate(PyQt_PyObject)"), self.showLastUpdate)
			checkUpdate.start()
			
		if (self.sendStartSignal):
			self.depart_m()
			
			
	##########################################################
	#                    Buttons Actions                     #
	##########################################################		
	
	def selectPlatform(self, button):
		self.selectedPlatform = button
		self.fileList = self.fileListByPlatform[button]
		self.barre.reset()
		self.barre.setRange(0, 100)
		self.barre.setValue(0)
		# FIXME: affichier le bonne barre de progression
		if button in self.telech:
			if self.telech[button]!=None and self.telech[button].isRunning():
				if self.fileListByPlatform[button][self.telech[button].row][3] == 0:
					self.barre.setRange(0, 0)
					self.barre.setValue(0)
					self.barreNull = 1
				else:
					self.barre.setRange(0, 100)
					self.barre.setValue(self.fileListByPlatform[button][self.telech[button].row][3])
					self.barreNull = 1
			else:
				self.barre.setValue(0)
		self.setTable()
		self.labelInfo.setText(self.textToShow[button])
		self.labelInfo2.setText(self.textToShow2[button])

	# Action de lancement du téléchargement
	#========================================================================
	def depart_m(self):
		if self.telech=={} or self.telech==None:
			if len(self.fileList)>0:
				# initialisation de la barre de progression
				self.barre.reset()
				self.barre.setRange(0, 0)
				self.barre.setValue(0)
				self.msgAbort=False
				self.trayIcon.setIcon(QIcon(os.path.join(self.execDir, "icons/systray/BotMU-wait.png")))
				# remise a zero du status
				count=0
				for val in self.fileList:
					if len(self.fileList[count])>4:
						self.fileList[count] = self.fileList[count][0:-1]
					if self.fileList[count][3] != 100:
						self.fileList[count][2] = os.path.join(self.execDir, "icons/wait.png")
						if self.fileList[count][3] == -1:
							self.fileList[count][3] = 0
					count+=1
				self.refreshTable()
				# démarre le téléchargement
				destination = unicode(self.fichier.text())
				self.destination = destination
				if not os.path.exists(destination) or not os.path.isdir(destination):
					os.makedirs(destination)
				
				
				for platform in self.supportedPlatform:
					if(len(self.fileListByPlatform[platform])>0):
						self.telech[platform] = self.supportedPlatform[platform][5].Telecharger(self.fileListByPlatform[platform], destination, parent=self)
					
						self.connect(self.telech[platform], SIGNAL("infotelech(PyQt_PyObject)"), self.infotelech)
						self.connect(self.telech[platform], SIGNAL("fintelech(PyQt_PyObject)"), self.fintelech)
						self.telech[platform].start()
		else:
			for platform in self.supportedPlatform:
				if platform in self.telech:
					if self.telech[platform]!=None and self.telech[platform].isRunning():
						if self.telech[platform].httpreq.pause:
							self.fileListByPlatform[platform][self.telech[platform].downloadingFile][2] = os.path.join(self.execDir, "icons/downloading.png")
							if platform == self.selectedPlatform:
								self.fileList[self.telech[platform].downloadingFile][2] = os.path.join(self.execDir, "icons/downloading.png")
								self.refreshTable()
							self.telech[platform].pausetelech()
							


	# Action de lancement du téléchargement
	#========================================================================
	def pause_m(self):
		if self.telech!={} and self.telech!=None:
			for platform in self.supportedPlatform:
				if platform in self.telech:
					if self.telech[platform]!=None and self.telech[platform].isRunning():
						if self.telech[platform].httpreq.pause:
							self.fileListByPlatform[platform][self.telech[platform].downloadingFile][2] = os.path.join(self.execDir, "icons/downloading.png")
							if platform == self.selectedPlatform:
								self.fileList[self.telech[platform].downloadingFile][2] = os.path.join(self.execDir, "icons/downloading.png")
						else:
							self.fileListByPlatform[platform][self.telech[platform].downloadingFile][2] = os.path.join(self.execDir, "icons/pause.png")
							if platform == self.selectedPlatform:
								self.fileList[self.telech[platform].downloadingFile][2] = os.path.join(self.execDir, "icons/pause.png")
						self.telech[platform].pausetelech()
						self.refreshTable()
				
	# Action d'arrèt du thread de téléchargement
	#========================================================================
	def stop_m(self):
		"""demande l'arrêt du téléchargement avant la fin"""
		if self.telech!={} and self.telech!=None:
			for platform in self.supportedPlatform:
				if platform in self.telech:
					if self.telech[platform]!=None and self.telech[platform].isRunning():
						if self.telech[platform].httpreq.pause:
							self.fileList[self.telech[platform].downloadingFile][2] = os.path.join(self.execDir, "icons/downloading.png")
							self.telech[platform].pausetelech()
						self.telech[platform].stoptelech()
	
	# Action de selection de répertoire
	#========================================================================
	def selectDir_m(self):
		dirPath = QFileDialog.getExistingDirectory(self, u"Dossier de téléchargement", self.userPersonalPath)
		if dirPath != "":
			if platform.system() == 'Windows':
				self.fichier.setText(dirPath.replace('/','\\'))
			else:
				self.fichier.setText(dirPath)


	def addLink(self, url, filename, continuedl=0):
		item = "icons/wait.png"
		platformFound = False
		for downloadPlatform in self.supportedPlatform:
			if not platformFound:
				if self.supportedPlatform[downloadPlatform][1] in url:
					fileinfos = [url, filename, item, continuedl]
					self.fileListByPlatform[downloadPlatform].append(fileinfos)
					platformFound = True
			self.setTable()

	# Action d'ajout d'un lien à télécharger
	#========================================================================
	def addLink_m(self):
		(url, ok) = QInputDialog.getText(self, self.tr("Entrer un lien MegaUpload"), self.tr("URL"))
		if (ok):
			(filename, ok2) = QInputDialog.getText(self, self.tr("Enter le nom du fichier"), self.tr("Filename"))
			if (ok2):
				self.addLink(url, filename)

	# Action d'ajout d'une liste de liens à télécharger
	#========================================================================
	def addList_m(self):
		listFilePath = QFileDialog.getOpenFileName(self, u'Liste de téléchargement', self.userPersonalPath, "*.bmu")
		if (listFilePath != "" and os.path.exists(listFilePath)):
			listFile = open(listFilePath, "r")
			data = listFile.read().decode("utf8")
			listFile.close()
			listLink = data.split("\n")
			if(len(listLink)>0):
				self.fileListString = list()
				for link in listLink:
					if(link!=""):
						fileInfos = link.split("::")
						self.addLink(fileInfos[0], fileInfos[1])

	# Action d'édition de la valeur séléctionée
	#========================================================================
	def editValue_m(self):
		items = self.listView.selectedIndexes()
		itemRow = items[0].row()
		itemColumn = items[0].column()
		itemValue = items[0].data().toString()
		(value, ok) = QInputDialog.getText(self, self.tr("Entrer un lien MegaUpload"), self.tr("URL"), QLineEdit.Normal, self.tr(itemValue))
		if(ok and value!=""):
			self.fileList[itemRow][itemColumn] = value
			self.setTable()

	# Action de suppression de la ligne séléctionnée
	#========================================================================
	def delRow_m(self):
		items = self.listView.selectedIndexes()
		deleteRow = QMessageBox.question(self, u" Supprimer " , u"Supprimer les lignes séléctionnées ?", QMessageBox.Yes | QMessageBox.No)
		if(deleteRow == QMessageBox.Yes):
			for item in items:
				itemRow = items[0].row()
				#lorsque le téléchargement tourne déjà:
				if self.telech!={} and self.telech!=None and self.selectedPlatform in self.telech and self.telech[self.selectedPlatform]!=None and self.telech[self.selectedPlatform].isRunning():
					#Si on supprime le lien en cours de téléchargement
					if itemRow == self.telech[self.selectedPlatform].row:
						self.telech[self.selectedPlatform].stopthisfile()
						self.fileList.pop(itemRow)
						self.telech[self.selectedPlatform].row-=1
					else:
						self.fileList.pop(itemRow)
						self.telech[self.selectedPlatform].row-=1
				#Lorsque le téléchargement est arreté:
				else:
					self.fileList.pop(itemRow)
			self.setTable()

	# Action de sauvegarde de la liste de téléchargement actuelle
	#========================================================================
	def saveList_m(self):
		saveInfos = ""
		for fileinfo in self.fileList:
			saveInfos += "%s::%s\n" % (fileinfo[0], fileinfo[1])
		saveFile = QFileDialog.getSaveFileName(self, u'Sauvegarde', self.userPersonalPath, "*.bmu")
		if saveFile!="":
			if(saveFile[-4:len(saveFile)]!=".bmu"):
				saveFile += ".bmu"
			try:
				sf = open(saveFile, "w+")
				sf.write(saveInfos.encode("utf8"))
				sf.close()
				QMessageBox.information(self, u"Enregistrement", u"Liste enregistrée")
			except:
				logError("mainWidget - saveList_m", sys.exc_info()[0], sys.exc_info()[1])
				QMessageBox.critical(self, u"Enregistrement", u"Impossible d'enregistrer dans ce fichier")

	# Action de suppression de tous les liens
	#========================================================================
	def removeAllLink_m(self):
		deleteRow = QMessageBox.question(self, u" Supprimer " , u"Supprimer toute la liste de téléchargement ?", QMessageBox.Yes | QMessageBox.No)
		if(deleteRow == QMessageBox.Yes):
			self.fileListByPlatform 		= {
				"Megaupload":[],
				"Youtube":[],
				"Classique":[]
			}
			self.fileList = []
			self.setTable()
	
	# Action de repérage de séléction d'une case du tableau
	#========================================================================
	def tableclick_m(self):
		if(self.listView.selectedIndexes() == []):
			self.editLink.setEnabled(False)
			self.delLink.setEnabled(False)
		else:
			items = self.listView.selectedIndexes()
			#S'il n'y a qu'un seul item de selectionné
			if len(items)==1 and items[0].data().toString()!='':
				self.item = items[0].row()
				self.editLink.setEnabled(True)		# Edition possible
				self.delLink.setEnabled(True)		# Suppression possible
			else:
				self.editLink.setEnabled(False)
				self.delLink.setEnabled(True)
				
				
				
	def openDirInNavigator_m(self):
		openDirInNavigator(unicode(self.fichier.text()))
		
	#########################################################
	#                    Others Windows                     #
	#########################################################		
				
				
	# Action d'ouverture de la fenètre "about"
	#========================================================================
	def showAbout_m(self):
		aboutText = "<div>"
		aboutText+= "	<center>"
		aboutText+= "		<img src='%s' alt='erf' />" % os.path.join(self.execDir, "icons/BotMU.png")
		aboutText+= "		<h1>BotMU %s</h1>" % self.parent.version
		aboutText+= "		<p>%s</p><br/>" % self.aboutDescription
		aboutText+= "	</center>"
		aboutText+= "	<p>Auteur: %s</p>" % self.aboutAuthor
		aboutText+= "	<p>Alias: %s</p>" % self.aboutAlias
		aboutText+= "	<p>Mail: %s</p>" % self.aboutMail
		aboutText+= "	<p>Website: <a href='%s'>%s</a></p>" % (self.aboutWebSite, self.aboutWebSite)
		aboutText+= "	<br/>"
		aboutText+= "</div>"
		
		aboutWindow = QMessageBox(self)
		aboutWindow.setWindowTitle("About BotMU...")
		aboutWindow.setText(aboutText)
		aboutWindow.exec_()
		
	# Action d'ouverture de la fenètre d'options
	#========================================================================
	def showOptions_m(self):
		options = Options(self)
		options.exec_()
			
	# Récéption et affichage d'information sur l'update
	#========================================================================
	def showLastUpdate(self, infos):
		updateVersion, updateLink, updateChangelog = infos
	
		updateText = u"<center>"
		updateText+= u"	<b>Une mise à jour est disponible</b>"
		updateText+= u"</center><br/>"
		updateText+= u"Informations:<br/><br/>"
		updateText+= u"* Votre version: %s<br/>" % self.parent.version
		updateText+= u"* Vesion actuelle: %s<br/><br/>" % updateVersion
		updateText+= u"Voir la différence:<br/><a href='%s'>%s</a><br/>" % (updateChangelog, updateChangelog)
	
		msgBox = QMessageBox(self)
		msgBox.setWindowTitle(u"Mise à jour ?")
		msgBox.setIcon(QMessageBox.Question)
		msgBox.setText(updateText)
		msgBox.addButton(u"Annuler", QMessageBox.NoRole)
		msgBox.addButton(u"Mise à jour", QMessageBox.YesRole)
		reply = msgBox.exec_()
		if(reply):
			if(platform.system()=="Windows"):
				subprocess.Popen("BotMUUpdater.exe %s" % updateLink)
				pass
			else:
				subprocess.Popen("python BotMUUpdater.py %s" % updateLink)
			sys.exit()
			
				
	##########################################################
	#                     Thread Functions                   #
	##########################################################
				
	# Récéption et affichage d'information du téléchargement
	#========================================================================
	def infotelech(self, msg):
		"""lancé à chaque réception d'info sur la progression du téléchargement"""
		telechbloc, taillebloc, totalblocs, msgInf, row, filename, platform = msg
		

		# Première execution pour cette platforme:
		if platform not in self.telechInit or self.telechInit[platform]!=True:
			self.telechInit[platform]=True
			self.downloadSpeedByPlatform[platform]= 0
			self.lasttime[platform] = time.time()
			self.downloadedOctets[platform] = 0
			self.execTime[platform] = 0.0
			
		# Affichage d'un message:
		if totalblocs == 0:
			if self.fileListByPlatform[platform][row][1]!=filename:
				self.fileListByPlatform[platform][row][1]=filename
				if platform == self.selectedPlatform: self.fileList[row][1]=filename
			
			#Si le message d'information viens de la platform: on affiche le message
			self.textToShow[platform] = u"%s" % (msgInf[0])
			self.textToShow2[platform] = ""
			if self.selectedPlatform == platform:
				self.setTable()
				if self.barre.maximum > 0 and self.barreNull != 1:
					self.barre.reset()
					self.barre.setRange(0, 0)
					self.barre.setValue(0)
					self.barreNull = 1
				self.labelInfo.setText(self.textToShow[platform])
				self.labelInfo2.setText(self.textToShow2[platform])
		# Boucle de téléchargement
		else:
			if platform == self.selectedPlatform:
				if self.barreNull == 1:
					self.barre.setRange(0, 100)
					self.barreNull = 0
			if(self.execTime[platform]>=3):
				#Calcul de la vitesse de téléchargement
				self.downloadSpeedByPlatform[platform] = int((self.downloadedOctets[platform]/3)/1024)
				#Calcul du temps restant estimé:
				timeLeftSec = int(float(totalblocs-(telechbloc*taillebloc))/float(self.downloadedOctets[platform]/3))
				#Calcul de la progression:
				progression = int(float(telechbloc)*float(taillebloc)/float(totalblocs)*100)
				self.fileListByPlatform[platform][row][3]=progression
				self.fileListByPlatform[platform][row][2]=os.path.join(self.execDir, 'icons/downloading.png')
				self.textToShow[platform] = u"%s" % (msgInf[0])
				self.textToShow2[platform] = u"* Fichier: %s/%s Mo\n* Debit: %s Ko/s\n* Temps restant estimé: %s" % (int((telechbloc*taillebloc)/(1024*1024)), int(totalblocs/(1024*1024)), self.downloadSpeedByPlatform[platform], secToTime(timeLeftSec))
				
				if not self.parent.isHidden():
					if platform == self.selectedPlatform:
						self.fileList[row][3]=progression
						self.fileList[row][2]=os.path.join(self.execDir, 'icons/downloading.png')
						self.barre.setValue(progression)
						self.labelInfo.setText(self.textToShow[platform])
						self.labelInfo2.setText(self.textToShow2[platform])
						self.refreshTable()
				totalspeed = 0
				for platformspeed in self.downloadSpeedByPlatform:
					totalspeed=totalspeed+int(self.downloadSpeedByPlatform[platformspeed])
				self.trayIcon.setIcon(QIcon(os.path.join(self.execDir, "icons/systray/BotMU-downloading.png")))
				self.trayIcon.setToolTip(u"BotMU - Télécharghement en cours\nVitesse totale: %s Ko/s" % totalspeed)
					
				self.saveSession()
				#On repart a 0
				self.execTime[platform]=0.0
				self.downloadedOctets[platform] = 0
			else:
				timeExe = ((time.time())-self.lasttime[platform])
				self.downloadedOctets[platform] = self.downloadedOctets[platform]+taillebloc
				self.execTime[platform]=self.execTime[platform]+timeExe
		self.lasttime[platform] = time.time()


	# Récéption et affichage d'information sur la FIN du téléchargement
	#========================================================================
	def fintelech(self, msg):
		"""Lancé quand le thread se termine (normalement ou pas)"""
		
		messagefin, row, platform = msg
		
		# Enregistrement des informations
		## Fin de téléchargement d'un fichier
		if(messagefin ==u"Téléchargement terminé\n\n"):
			self.fileListByPlatform[platform][row][2] = os.path.join(self.execDir, "icons/ok.png")
			
			self.fileListByPlatform[platform][row][3]=100
			self.textToShow[platform] = u"Fin du téléchargement de %s" % self.fileListByPlatform[platform][row][1]
			self.textToShow2[platform] = ""
		else:
			self.fileListByPlatform[platform][row][2] = os.path.join(self.execDir, "icons/error.png")
			self.fileListByPlatform[platform][row].append(messagefin)
			self.textToShow[platform] = u"%s : %s"  % (messagefin, self.fileListByPlatform[platform][row][1])
			self.textToShow2[platform] = ""
			
		## Fin de téléchargement d'une liste de fichier
		if(row==len(self.fileListByPlatform[platform])-1):
			self.textToShow[platform] = u"Fin du téléchargement de la liste %s" % platform
			self.textToShow2[platform] = ""
			self.trayIcon.setToolTip(self.textToShow[platform])
			self.trayIcon.setIcon(QIcon(os.path.join(self.execDir, "icons/systray/BotMU-ok.png")))
			self.trayIcon.showMessage(platform, self.textToShow[platform], QSystemTrayIcon.Information, 2000)
			if platform in self.telech:
				self.telech.pop(platform)
			if len(self.telech)==0:
				QMessageBox.information(self, u"Téléchargement", u"Fin du téléchargement des fichiers")
			
		# Affichage des information si le message d'info concerne la plateforme séléctionnée
		if platform == self.selectedPlatform:
			self.barre.reset()
			self.barre.setRange(0, 100)
			self.barreNull = 0
			self.barre.setValue(self.fileListByPlatform[platform][row][3])
			self.labelInfo.setText(self.textToShow[platform])
			self.labelInfo2.setText(self.textToShow2[platform])

		
	##########################################################
	#                     Other Functions                    #
	##########################################################
		
	# Action de lecture des options enregistrée
	#========================================================================
	def readOptions(self):
	
		setupFile = os.path.join(self.configDir, "setup.ini")
		#s'il n'existe pas, on le créé
		if not os.path.isfile(setupFile):
			try:
				paramfile = open(setupFile, "w+")
				paramfile.close()
			except:
				logError("mainWidget - init - 1", sys.exc_info()[0], sys.exc_info()[1])
		
		paramfile = open(setupFile, "r")
		param = (paramfile.read()).split("\n")
		for item in param:
			if item.split("=")[0] == 'proxy':
				self.proxy = item.split("=")[1]
			if item.split("=")[0] == 'verbose':
				if item.split("=")[1] == 'True':
					self.verbose = True
				else:
					self.verbose = False
			if item.split("=")[0] == 'destination':
				self.defaultDir = item.split("=")[1].decode("utf8")
			if item.split("=")[0] == 'checkupdate':
				self.checkUpdate = int(item.split("=")[1].decode("utf8"))
			if item.split("=")[0] == 'checkupdatedelay':
				self.checkupdateDelay = int(item.split("=")[1].decode("utf8"))
			if item.split("=")[0] == 'checkupdatelast':
				self.checkupdateLast = int(item.split("=")[1].decode("utf8").split(".")[0])
			if item.split("=")[0] == 'limit':
				self.limit = int(item.split("=")[1].decode("utf8").split(".")[0])


	# creation du tableau
	#========================================================================
	def setTable(self):
		self.listView.reset()
		self.listView.setModel(MyTableModel(self.fileList, self.tmheader, self))
		self.listView.setIconSize(QSize(32, 32))
		self.listView.setEditTriggers(QAbstractItemView.NoEditTriggers)
		self.connect(self.listView.selectionModel(), SIGNAL("selectionChanged(QItemSelection, QItemSelection)"), self.tableclick_m)
		
		self.listView.setItemDelegateForColumn(2,ProgressBarDelegate(self, self.fileList))
		#self.listView.setItemForColumn(1,QStandardItem("test"))
		self.listView.resizeColumnToContents(0)
		self.listView.resizeColumnToContents(1)
		
		if self.listView.size().width() < 880:
			thirdColumnWidth = 880 - (self.listView.columnWidth(0) + self.listView.columnWidth(1) + 45)
		else:
			thirdColumnWidth = self.listView.size().width() - (self.listView.columnWidth(0) + self.listView.columnWidth(1) + 45)
		self.listView.setColumnWidth(2, thirdColumnWidth)
		
		
		#self.listView.setColumnWidth(2, 380)
		

	# mise a jour du tableau
	#========================================================================
	def refreshTable(self):
		self.listView.setItemDelegateForColumn(2,ProgressBarDelegate(self, self.fileList))
		
		if self.listView.size().width() < 880:
			thirdColumnWidth = 880 - (self.listView.columnWidth(0) + self.listView.columnWidth(1) + 45)
		else:
			thirdColumnWidth = self.listView.size().width() - (self.listView.columnWidth(0) + self.listView.columnWidth(1) + 45)
		self.listView.setColumnWidth(2, thirdColumnWidth)
	
	# Fonction de repérage d'un lien dans le presse papier
	#========================================================================
	def clipboardChanged(self):
		tmpError = False
		newLinkFound = []
		platformFound = False
		try:
			test = chardet.detect(self.clipboard.text())
			tmp = self.clipboard.text("plain", QClipboard.Selection)
		except:
			try:
				tmp = unicode(self.clipboard.text(), "ISO-8859-15")
			except:
				tmp = ""
				logError("mainWidget - clipboardChanged - 1", sys.exc_info()[0], sys.exc_info()[1])
				tmpError = True
		if not tmpError:
			self.clipboardValue = tmp
			try:
				for downloadPlatform in self.supportedPlatform:
					if not platformFound:
						if self.supportedPlatform[downloadPlatform][1] in self.clipboardValue:
							links = self.clipboardValue.split(self.supportedPlatform[downloadPlatform][1])
							for link in links:
								if(len(link)>=self.supportedPlatform[downloadPlatform][3] and len(link)>0):
									if(self.supportedPlatform[downloadPlatform][3]!=0):
										link = link[0:self.supportedPlatform[downloadPlatform][3]]
									link = link.split(" ")[0].split('"')[0].split("'")[0]
									patern = re.compile(self.supportedPlatform[downloadPlatform][4])
									if(patern.match(link)):
										link = "%s%s" % (self.supportedPlatform[downloadPlatform][1], link)
										if link[0:7]!= "http://":
											link = "http://%s" % link
										ok = QMessageBox.question(self, u" Nouveau lien repéré " , "Ajouter %s ?" % link, QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
										if(ok == QMessageBox.Yes):
											self.addLink(link, "auto")
											platformFound = True
			except:
				logError("mainWidget - clipboardChanged - 2", sys.exc_info()[0], sys.exc_info()[1])
					

	# Fonction de sauvegarde de la session (téléchargement en attente et en cours)
	#========================================================================
	def saveSession(self):
		saveInfos = ""
		saveInfos += "%s\n" % self.fichier.text()
		for platform in self.supportedPlatform:
			for fileinfo in self.fileListByPlatform[platform]:
				if fileinfo[2]!= os.path.join(self.execDir, "icons/ok.png"):
					if fileinfo[3] == -1:
						fileinfo[3] = 0
					saveInfos += "%s::%s::%s\n" % (fileinfo[0], fileinfo[1], fileinfo[3])
		try:
			saveFile = os.path.join(self.configDir, "lastsession.bsav")
			sf = open(saveFile, "w+")
			sf.write(saveInfos.encode("utf8"))
			sf.close()
		except:
			logError("mainWidget - saveSession", sys.exc_info()[0], sys.exc_info()[1])
	
	# Fonction de vérification de l'existance d'une ancienne session sauvegardée
	#========================================================================
	def checkIfLastSessionSaved(self):
		lastsessionFile = os.path.join(self.configDir, "lastsession.bsav")
		if(os.path.exists(lastsessionFile)):
			try:
				listFile = open(lastsessionFile, "r")
				data = listFile.read().decode("utf8")
				listFile.close()
			except:
				pass
		else:
			data = ""
		
		if (len(data.split("\n")) > 1):
			reply = QMessageBox.question(self, u"Enregistrement détécté", u"Le téléchargement à été avorté lors de la derniere session,\nSouhaitez vous reprendre ces téléchargements ?", QMessageBox.No|QMessageBox.Yes, QMessageBox.Yes)
			if(reply==QMessageBox.Yes):
				try:
					listFile = open(lastsessionFile, "r")
					data = listFile.read().decode("utf8")
					listFile.close()
				except:
					logError("mainWidget - checkIfLastSessionSaved - 1", sys.exc_info()[0], sys.exc_info()[1])
				listLink = data.split("\n")
				if(len(listLink)>=1):
					self.fileListString = list()
					self.defaultDir = listLink[0]
					for link in listLink[1:len(listLink)]:
						if(link!=""):
							fileInfos = link.split("::")
							fileInfos[2] = int(fileInfos[2])
							if fileInfos[2] == -1:
								fileInfos[2] = 0 
							self.addLink(fileInfos[0], fileInfos[1], fileInfos[2])
					self.sendStartSignal = True
			else:
				try:
					sf = open(lastsessionFile, "w+")
					sf.close()
				except:
					logError("mainWidget - checkIfLastSessionSaved - 2", sys.exc_info()[0], sys.exc_info()[1])
